( cd usr/include/netinet ; rm -rf tcp.h )
( cd usr/include/netinet ; ln -sf  ip_tcp.h tcp.h )
( cd usr/include/netinet ; rm -rf udp.h )
( cd usr/include/netinet ; ln -sf  ip_udp.h udp.h )
( cd usr/include ; rm -rf asm )
( cd usr/include ; ln -sf  /usr/src/linux/include/asm asm )
( cd usr/include ; rm -rf linux )
( cd usr/include ; ln -sf  /usr/src/linux/include/linux linux )
( cd usr/include/netinet ; rm -rf in_systm.h )
( cd usr/include/netinet ; ln -sf  in_system.h in_systm.h )
