/* AUTOMATICALLY GENERATED; DO NOT EDIT! */ 
#ifndef _G_config_h
#define _G_config_h

#define _LINUX_C_LIB_VERSION		"4.5.26"
#define _LINUX_C_LIB_VERSION_MAJOR	4
#define _LINUX_C_LIB_VERSION_MINOR	5
#define _LINUX_C_LIB_VERSION_SUBMINOR	26

#define _G_LIB_VERSION "0.63"
#define _G_NAMES_HAVE_UNDERSCORE 1
#define _G_HAVE_ST_BLKSIZE 1
typedef long _G_clock_t;
typedef unsigned short _G_dev_t;
typedef long int _G_fpos_t;
typedef unsigned short _G_gid_t;
typedef unsigned long _G_ino_t;
typedef unsigned short _G_mode_t;
typedef unsigned short _G_nlink_t;
typedef long _G_off_t;
typedef int _G_pid_t;
typedef int _G_ptrdiff_t;
typedef unsigned int _G_sigset_t;
typedef unsigned int _G_size_t;
typedef long _G_time_t;
typedef unsigned short _G_uid_t;
typedef long int _G_wchar_t;
typedef int /* default */ _G_int32_t;
typedef unsigned int /* default */ _G_uint32_t;
typedef int _G_ssize_t;
typedef void * _G_va_list;
#define _G_signal_return_type void
#define _G_sprintf_return_type int
#define _G_BUFSIZ 1024 
#define _G_FOPEN_MAX  256 
#define _G_FILENAME_MAX  1024 
#define _G_NULL 0 /* default */
#define _G_ARGS(ARGLIST) (...)
#define _G_HAVE_ATEXIT 1
#define _G_HAVE_SYS_RESOURCE 1
#define _G_HAVE_SYS_SOCKET 1
#define _G_HAVE_SYS_WAIT 1
#define _G_HAVE_UNISTD 1
#define _G_HAVE_DIRENT 1
#define _G_HAVE_CURSES 1
#define _G_MATH_H_INLINES 0
#endif /* !_G_config_h */
